function [Phi_draws, Sigma_draws,constant_draws] = GIBBS_VAR(data,nlag,ndraws,burn_in,add_constant)

% Purpose: 
% Bayesian inference of the VAR system
% -----------------------------------
% Model:
% Y(t) = c  +  Phi(1)*Y(t-1)  + ... +  Phi(p)*Y(t-p)  +  ut
% where ut ~ MVN(0,Sigma), Y(t) contains d component variables
% -----------------------------------
% Algorithm: 
% Rearrange dependent variables and regressors
% Bayesian VAR model is identical to SUR model
% Gibbs sampler with Lindley and Smith (1972) proper priors
% Phi ~ N(mu,V), Sigma ~ W(Omega,df)
% Hyperparameters, draws and burn-in are specified below
% You may change them if needed
% -----------------------------------
% Usage:
% data = regressors in the VAR system
% nlag = number of lags (p) in the VAR(p) system
% ndraws = number of draws in MCMC
% burn_in = number of burn-in draws in MCMC
% add_constant = whether to add a constant to the VAR (default = 1)
% -----------------------------------
% Returns:
% Phi_draws = posterior draws of phi(1),,phi(p), d*d*p*R 4-dimension array
% Sigma_draws = posterior draws of covariance matrix, d*d*R 3-dimension array
% constant_draws = posterior draws of the intercept in the VAR model (not the mean)
% -----------------------------------
% 
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


if nargin<2,    error('Incomplete data');      end
if nargin<3,    ndraws = 10000;                                 end
if nargin<4,    burn_in = ndraws*0.5;                            end
if nargin<5,    add_constant = 1;                              end


try
    JustTest = gamrnd(3,1);
catch
    disp(' ')
    disp('Oooopse, Matlab statistics toolbox is not installed.')
    disp('You may download a compatibility package on my website.')
    disp('http://www.public.iastate.edu/~hqi/toolkit')
    error('Program exits.')
end

% Analyze data
constant_check = all(data == 1);
data(:,constant_check) = [];

[nobs,dim] = size(data);
nobs = nobs - nlag;
nreg_eq = dim * nlag + add_constant;
nreg_sum = dim * nreg_eq;

% Constuct data matrix for SUR regression
% X_stack is the common regressors of every equation
X_stack = zeros(nobs,nreg_eq);
if add_constant == 0       
    for p = 1:nlag
        X_stack(:, (p-1)*dim+1 : p*dim) = data(nlag-p+1 : end-p, :);
    end
else
    X_stack(:,1) = ones(nobs,1);
    for p = 1:nlag
        X_stack(:, (p-1)*dim+2 : p*dim+1) = data(nlag-p+1 : end-p, :);
    end
end

% X_SUR is the pooled regressors (stack eq. -> stack obs.)
% but it is easer to stack in the reverse way (stack obs. -> stack eq.)
% then transform to (stack eq. -> stack obs.)
X_flag = repmat('X_stack,',1,dim);
X_flag = X_flag(1:end-1);
X_SUR = eval(['blkdiag(',X_flag,')']);
X_temp1 = zeros(nobs,dim);
for m = 1:nreg_sum
    X_temp1(:) = X_SUR(:,m);
    X_temp2 = X_temp1';
    X_SUR(:,m) = X_temp2(:);
end

% Y_SUR is the pooled dependent variable (stack eq. -> stack obs.)
Y_mat = data(nlag + 1:end,:)';
Y_SUR = Y_mat(:);


%----------------------------------------
% Prior distribution settings
%  Beta ~ N(mu,V)
%  Sigma ~ W(,df)
% You may change the hyperparameters here if needed
prior_mu = zeros(nreg_sum,1);
prior_V = 100*eye(nreg_sum);
prior_omega = eye(dim);
prior_df = 10;
%-----------------------------------------


% Preallocate memory
Beta_draws = zeros(nreg_sum,ndraws-burn_in);
Sigma_draws = zeros(dim,dim,ndraws-burn_in);
resid_matrix = zeros(dim,nobs);
Beta_use = prior_mu + chol(prior_V)'*randn(nreg_sum,1);
inv_Sigma = WISHART_RND(prior_omega,prior_df);
X_SUR_sd = zeros(dim*nobs,nreg_sum);
inv_prior_V = inv(prior_V);
inv_prior_V_mu = inv_prior_V*prior_mu;
inv_prior_omega = inv(prior_omega);


% Gibbs sampler begins
H_status_bar = waitbar(0,'MCMC in progress    0%');
percentage = floor(ndraws/100);
for r = 1:ndraws
    
    if mod(r,percentage) == 0
        waitbar(r/ndraws,H_status_bar,['MCMC in progress    ',num2str(floor(r/ndraws*100)),'%']);
    end
    
    % Conditional posterior Beta follows N(Dd,D)
    % Normalize data so that the codes runs faster
    P = chol(inv_Sigma);
    Y_mat_sd = P * Y_mat;
    Y_SUR_sd = Y_mat_sd(:);          
    for n = 1:dim
        for s = 1:n
            X_SUR_sd(s:dim:end, (n-1)*nreg_eq+1 : n*nreg_eq) = P(s,n) .* X_stack;
        end
    end
         
    beta_prec = X_SUR_sd'*X_SUR_sd + inv_prior_V;
    beta_mean = beta_prec \ (X_SUR_sd'*Y_SUR_sd + inv_prior_V_mu);   
    Beta_use = beta_mean + chol(beta_prec) \ randn(nreg_sum,1);
    
    % Conditional posterior Sigma follows inverse Wishart
    resid = Y_SUR - X_SUR * Beta_use;    
    resid_matrix(:) = resid;    
    RSS_matrix = resid_matrix * resid_matrix';
    post_omega = inv(inv_prior_omega + RSS_matrix);
    inv_Sigma = WISHART_RND(post_omega,nobs + prior_df);
        
    if r > burn_in
        Beta_draws(:,r - burn_in) = Beta_use;
        Sigma_draws(:,:,r - burn_in) = inv(inv_Sigma);
    end      
end


% Extract Phi from Beta
Phi_draws = zeros(dim,dim,nlag,ndraws-burn_in);
if add_constant == 1
    constant_draws = zeros(dim,ndraws-burn_in);    
    beta_matrix = zeros(dim*nlag + 1,dim);
    for m = 1:ndraws-burn_in
        beta_matrix(:) = Beta_draws(:,m);
        beta_matrix_tran = beta_matrix';
        constant_draws(:,m) = beta_matrix_tran(:,1);
        for n = 1:nlag
            Phi_draws(:,:,n,m) = beta_matrix_tran(:,dim*(n-1) + 2:dim*n + 1);
        end
    end    
else
    constant_draws = [];
    beta_matrix = zeros(dim*nlag,dim);
    for m = 1:ndraws-burn_in
        beta_matrix(:) = Beta_draws(:,m);
        beta_matrix_tran = beta_matrix';        
        for n = 1:nlag
            Phi_draws(:,:,n,m) = beta_matrix_tran(:,dim*(n-1) + 1:dim*n);
        end
    end
    
end

% Summarize posterior distribution
Phi_mean = mean(Phi_draws,4);
Sigma_mean = mean(Sigma_draws,3);
constant_mean = mean(constant_draws,2);
eval([char([81 72 49 61]),'[87 114 105 116 116 101 110 32 98 121];'])
eval([char([81 72 50 61]),'[32 72 97 110 103 32 81 105 97 110];'])

% Display results
if add_constant == 1
    disp('Constant')
    disp(constant_mean')
    disp(' ')
end

for p = 1:nlag
    disp(['Posterior Phi',num2str(p),' coefficients'])    
    for n = 1:dim
        fprintf('%8.2f', Phi_mean(n,:,p))
        fprintf('\n')
    end
    fprintf('\n')
    disp(' ')
end

disp('Posterior covariance matrix of the VAR system')
for m = 1:dim
    fprintf('%8.2f', Sigma_mean(m,:))
    fprintf('\n')
end
fprintf('\n')
fwrite(1, char([QH1,QH2,10,13]))
close(H_status_bar);
end

function sample = WISHART_RND(Omega,df)
dim = size(Omega,1);
P = chol(Omega);
Y = P' * randn(dim,round(df));
sample = Y * Y';
end
