function IRF = IRF_SHORT(Phi,Sigma,MA_lag,shock_flag,response_flag,max_draws)

% Purpose: 
% Compute impulse response function (IRF) with cholesky decomposition
% That is, impose one-way recursive restriction of zero short-run effects  
% The inputs arguments are parameter estimates of reduced form Bayesian VAR(p)
% -----------------------------------
% Usage:
% Phi = posterior draws of phi(1),...,phi(p), d*d*p*R 4-dimension array
% Sigma = posterior draws of covariance matrix, d*d*R 3-dimension array
% MA_lag = lags of VMA representation (periods of the IRF)
% shock_flag = the indexes of structure shocks
% response_flag = the indexes of response variables
% max_draws = maximum number of MCMC draws used to repeat the IRF
% -----------------------------------
% Returns:
% IRF = a four-dimension array containing the impulse response function
%     Dimension 1: the number of response variables (length of response_flag)
%     Dimension 2: the number of shock variables (length of shock_flag)
%     Dimension 3: lags of VMA representation (periods of the IRF)
%     Dimension 4: MCMC draws used to repeat the IRF
% -----------------------------------
%
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


[dim,dim1,nlag,ndraws] = size(Phi);

if nargin < 2;  error('Please use Phi and Sigma from the output of Gibbs sampler'); end
if nargin < 3;  MA_lag = 36;  end
if nargin < 4;  shock_flag = 1;  end
if nargin < 5;  response_flag = 1:dim;  end
if nargin < 6;  max_draws = 200; end

if ndraws > max_draws
    Phi = Phi(:,:,:,ndraws-max_draws+1:end);
    Sigma = Sigma(:,:,ndraws-max_draws+1:end);
    ndraws = max_draws;
end

% Invert VAR(p) into VMA(inf), the first hundred periods are blank
Psi = zeros(dim,dim,100+MA_lag,ndraws);
H_status_bar = waitbar(0,'Invert VAR into VMA  %');
percentage = floor(ndraws/100);
for r = 1:ndraws
    if mod(r,percentage) == 0
        waitbar(r/ndraws,H_status_bar,['Step 1/2: Invert VAR into VMA  ',num2str(floor(r/ndraws*100)),'%']);
    end
    
    Psi(:,:,100,r) = eye(dim);
    for m = 101:100+MA_lag
        for n = 1:nlag
            Psi(:,:,m,r) = Psi(:,:,m,r) + Phi(:,:,n,r) * Psi(:,:,m-n,r);
        end
    end      
end
close(H_status_bar);

% Use Cholesky decomposition to impose recursive short-run restrictions
% The first period of IRF is period zero (date of the current shock)
nshock = length(shock_flag);
nresponse = length(response_flag);
IRF = zeros(nresponse,nshock,MA_lag+1,ndraws);
H_status_bar = waitbar(0,'Step 2/3: Recover structural VMA  %');
for r = 1:ndraws
    
    if mod(r,percentage) == 0
        waitbar(r/ndraws,H_status_bar,['Step 2/2: Recover structural VMA  ',num2str(floor(r/ndraws*100)),'%']);
    end
    
    P = chol(Sigma(:,:,r),'lower');    
    for n = 1:MA_lag+1      
        IRF(:,:,n,r) = Psi(response_flag,:,n+99,r) * P(:,shock_flag);        
    end        
end
close(H_status_bar);

% Plot Impulse response function
if nargout == 0
    IRF_mean = mean(IRF,4);
    IRF_lb = prctile(IRF,2.5,4);
    IRF_ub = prctile(IRF,97.5,4);
    
    seq = (0:MA_lag)';
    nshock = length(shock_flag);
    nresponse = length(response_flag);
    count = 1;
    for m = 1:nshock
        for n = 1: nresponse
            mean_use = IRF_mean(n,m,:);
            ub_use = IRF_ub(n,m,:);
            lb_use = IRF_lb(n,m,:);
            % The blue solid line is the mean response
            % Two red dotted line is the 95% credible interval
            subplot(nresponse,nshock,count),plot(seq,mean_use(:),'-b',  seq,ub_use(:),':r',  seq,lb_use(:),':r')            
            title(['Response of Variable ',num2str(response_flag(n)),' to Shock ',num2str(shock_flag(m))])
            count = count + 1;
        end
    end    
end

