function [Phi_draws, Sigma_draws,constant_draws] = VARDAS1(Y,nlag,sum_flag,add_constant,ndraws,burn_in)

% Purpose: 
% Bayesian inference of varied frequency VAR(p) model (VARDAS), Version 1
% This version only supports data of two frequencies, balanced aggregation cycle
% Some variables at a high frequency and other variables at a low frequency
% For example: quarterly GDP and monthly CPI, M1, etc.
% Disaggregated data in an aggregation cycle are treated as a block
% and the Gibbs sampler moves block by block.
% This version is faster than VARDAS3 and VARDAS4
% -----------------------------------
% Model:
% The latent Y(t) contains d component variables and follows VAR(p)
% Y(t) = c + phi(1) * Y(t-1) + ...+ phi(p) * Y(t-p) + ui,
% where ui ~ N(0,Sigma)
% However, some of the Y(t) are aggregated or averaged
% -----------------------------------
% Algorithm: 
% Qian (2011) "Vector Autoregression with varied frequency data"
% Conditional on latent data, this is largely a linear regression model.
% Posterior latent variables follows MVN with linear constraints,
% which can be decomposed into a conditional normal and a degenerated distribution 
% -----------------------------------
% Usage:
% Y = data of VAR (T * d matrix, or d * T matrix) (See notes below)
% nlag = number of lags (p) in the VAR model
% sum_flag: specify how the latent high frequency data are grouped into low frequency data
%           1 = sum, 0 = average
% add_constant = whether to add a constant in the VAR regression
% ndraws = number of draws in MCMC
% burn_in = number of burn-in draws in MCMC
% -----------------------------------
% Returns:
% Phi_draws = posterior draws of phi(1),...,phi(p), d*d*p*R 4-dimension array
% Sigma_draws = posterior draws of covariance matrix, d*d*R 3-dimension array
% constant_draws = posterior draws of the intercept (not the mean), d*R matrix
% where R = ndraws - burn_in
% -----------------------------------
% Notes:
% How to provide the data:
% The time index 1,...,T is for highest frequency data
% If disaggregated (high frequency) data are not available, assign NaN
% Also record the aggregated data at the end of the aggregation cycle.
% For example,
% In a monthly VAR model, one variable only have quarterly data
% Suppose data in the first two quarters are [20 18],
% Then record the data as Y(1,:) = [NaN NaN 20 NaN NaN 18]
% The program will count a run of NaN in front of a data point
% to determine the aggregation structure
% 
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


if nargin < 2;  nlag = 1;   end;
if nargin < 3;  sum_flag = 1;   end;
if nargin < 4;  add_constant = 1;   end;
if nargin < 5;  ndraws = 10000;   end;
if nargin < 6;  burn_in = ndraws * 0.5;   end;

% Analyze data
[dim,nobs] = size(Y);
if dim > nobs
    Y = Y';
    [dim,nobs] = size(Y);
end
nreg_eq = dim * nlag + add_constant;
nreg_sum = nreg_eq * dim;

for t = nobs:-1:1
    if all(~isnan(Y(:,t)))
        if t < nobs
            disp(['It seems that the last ',num2str(nobs-t),' observations are incomplete. Discard.'])
            Y = Y(:,1:t);           
            nobs = t;
        end
        break
    end
end

% Analyze aggregation structure
miss_ind = isnan(Y);
agg_variable = any(miss_ind,2);
if ~any(agg_variable)
    miss_ind = (Y == 0);
    agg_variable = any(miss_ind,2);
    if any(agg_variable)
        disp('Missing observations not found. Treat 0 as missing value.')
    else
        disp('Cannot find any missing observation in the data.')
        disp('The program will run as if it is a standard VAR model,')
        disp('but it will go through many unnecessary computations.')
        agg_variable = false(dim,1);
    end
end

data_test = miss_ind(agg_variable,:);
nagg = round(1/mean(~data_test(:)));
if isnan(nagg)
    % This happens when there is no missing data
    nagg = 1;
end
disp(['Aggregation cycle: ',num2str(nagg)])
if ~all(all(data_test(:,1:nagg-1))) || any(data_test(:,nagg))
    disp('Unbalanced Aggregation: Series do not start from the aggregation cycle')    
    for t = 1:nagg*2
        if all(all(data_test(:,t:t+nagg-2))) && ~any(data_test(:,t+nagg-1))
            disp(['Automatic truncate the inital ',num2str(t-1),' observations'])
            Y = Y(:,t:end);
            [dim,nobs] = size(Y);
            miss_ind = isnan(Y);
            data_test = miss_ind(agg_variable,:);
            break
        end
    end
    if t == nagg*2
        error('Please use VARDAS3 or VARDAS4 in the package for unbalanced aggregation')
    end
end

for t = nagg:nagg:nobs
    if ~all(all(data_test(:,t-nagg+1:t-1))) || any(data_test(:,t))
        error(['Unbalanced Aggregation at Date ',num2str(t), ': Please use VARDAS3 or VARDAS4 in the package instead.'])
    end
end

% Explore the Markov property of the AR(p) process
% Extend the aggregation period so as to form unditional MVN
period1 = nlag;
period2 = nlag;
nagg_ext = nagg + period1 + period2;
for t = nagg:nagg:nobs
    if t-nagg-period1+1 > 0
        miss_ind_ext = miss_ind(:, t-nagg-period1+1 : t+period2);        
        break
    end
end
miss_ind_ext(:,1:period1) = false;
miss_ind_ext(:,end-period2+1:end) = false;
miss_ind_ext = miss_ind_ext(:);
intact_ind_ext = ~miss_ind_ext;
nmiss = sum(miss_ind_ext);

% Transform summed data into averaged data
if sum_flag == 1
    Y(agg_variable,:) = Y(agg_variable,:) ./ nagg;    
end
    

%----------------------------------------
% Prior distribution settings
%  Beta ~ N(mu,V)
%  Sigma ~ W(Omega,df)
% You may change the hyperparameters here if needed
prior_mu = zeros(nreg_sum,1);
prior_V = 100*eye(nreg_sum);
prior_omega = eye(dim);
prior_df = 10;
%-----------------------------------------

% Preallocate memory
Phi_draws = zeros(dim,dim,nlag,ndraws - burn_in);
Sigma_draws = zeros(dim,dim, ndraws - burn_in);
constant_draws = zeros(dim,ndraws - burn_in);

% Initial draws
Beta_use = prior_mu + chol(prior_V)'*randn(nreg_sum,1);
inv_Sigma = WISHART_RND(prior_omega,prior_df);

Y_sim = Y;
for d = 1:dim
    if agg_variable(d)
        for t = nagg:nagg:nobs
            Y_sim(d,t-nagg+1:t) = Y(d,t);            
        end
    end
end

% For faster codes
X_flag = repmat('X_stack,',1,dim);
X_flag = X_flag(1:end-1);
Phi_big = [zeros(dim,dim*nlag);[eye(dim*(nlag-1)),zeros(dim*(nlag-1),dim)]];
Sigma_big = zeros(dim*nlag,dim*nlag);
update_draws = zeros(sum(agg_variable),nagg-1);

% Construct aggregation transform matrix
A = eye(nagg_ext*dim);
for d = 1:dim
    if agg_variable(d)
        t = period1+nagg;
        loc_A = dim*(t-1)+d;
        A(loc_A,loc_A - dim .* (0:nagg-1)) = 1/nagg;
    end
end


% Gibbs sampler
H_status_bar = waitbar(0,'MCMC in progress 0%');
percentage = floor(ndraws / 100);
TotalTime = 0;
tic
for r = 1:ndraws
    
    % You are welcome to copy this status bar into your own codes,
    % but please also include the designer's info: Designed by Hang Qian    
    if mod(r,percentage) == 0
        Elapse = toc;
        TotalTime = TotalTime + Elapse;
        RemainTime = TotalTime * (ndraws - r) / r;
        if RemainTime > 3600
            RemainTime = round(RemainTime/3600);
            waitbar(r / ndraws,H_status_bar,['MCMC in Progress ',num2str(floor(r / ndraws * 100)),'%   Time Remaining ',num2str(RemainTime),' h']);
        elseif RemainTime > 60
            RemainTime = round(RemainTime/60);
            waitbar(r / ndraws,H_status_bar,['MCMC in Progress ',num2str(floor(r / ndraws * 100)),'%   Time Remaining ',num2str(RemainTime),' m']);
        else
            RemainTime = round(RemainTime);
            waitbar(r / ndraws,H_status_bar,['MCMC in Progress ',num2str(floor(r / ndraws * 100)),'%   Time Remaining ',num2str(RemainTime),' s']);
        end
        tic
    end
    
    % Config VAR regression data (1/3)
    % Y_SUR is the pooled dependent variable (stack eq. -> stack obs.)
    Y_mat = Y_sim(:, nlag+1:nobs);
    Y_SUR = Y_mat(:);
    
    % Config VAR regression data (2/3)
    % X_stack is the common regressors of every equation
    X_stack = zeros(nobs-nlag,nreg_eq);
    if add_constant == 0
        for p = 1:nlag
            X_stack(:,(p-1)*dim+1:p*dim) = Y_sim(:,nlag-p+1:nobs-p)';
        end
    else
        X_stack(:,1) = ones(nobs-nlag,1);
        for p = 1:nlag
            X_stack(:,(p-1)*dim+2:p*dim+1) = Y_sim(:,nlag-p+1:nobs-p)';
        end
    end
    
    % Config VAR regression data (3/3)
    % X_SUR is the pooled regressors (stack eq. -> stack obs.)
    % but it is easer to stack in the reverse way (stack obs. -> stack eq.)
    % then transform to (stack eq. -> stack obs.)
    X_SUR = eval(['blkdiag(',X_flag,')']);    
    X_temp1 = zeros(nobs-nlag,dim);
    for m = 1:nreg_sum
        X_temp1(:) = X_SUR(:,m);
        X_temp2 = X_temp1';
        X_SUR(:,m) = X_temp2(:);
    end
    
    % Conditional posterior Beta follows N(Dd,D)
    % Normalize data so that the codes runs faster
    P = chol(inv_Sigma);
    Y_mat_sd = P * Y_mat;
    Y_SUR_sd = Y_mat_sd(:);
    X_SUR_sd = zeros(dim*(nobs-nlag),nreg_sum);
    for n = 1:dim
        for s = 1:n
            X_SUR_sd(s:dim:end, (n-1)*nreg_eq+1:n*nreg_eq) = P(s,n) .* X_stack;
        end
    end         
    beta_prec = X_SUR_sd'*X_SUR_sd + inv(prior_V);
    beta_mean = beta_prec \ (X_SUR_sd'*Y_SUR_sd + prior_V \ prior_mu);   
    Beta_use = beta_mean + chol(beta_prec) \ randn(nreg_sum,1);
    
    % Recover Phi and constant from Beta
    Beta_mat = zeros(nreg_eq,dim);
    Beta_mat(:) = Beta_use;
    Beta_mat = Beta_mat';
    if add_constant == 1
        constant_use = Beta_mat(:,1);
        Beta_mat = Beta_mat(:,2:end);
    else
        constant_use = zeros(dim,1);
    end
    Phi_use = zeros(dim,dim,nlag);
    Phi_use(:) = Beta_mat(:);    
    
    % Conditional posterior Sigma follows inverse Wishart
    resid = Y_SUR - X_SUR * Beta_use;
    resid_matrix = zeros(dim,nobs-nlag);
    resid_matrix(:) = resid;    
    RSS_matrix = resid_matrix * resid_matrix';
    post_omega = inv(inv(prior_omega) + RSS_matrix);
    inv_Sigma = WISHART_RND(post_omega,nobs - nlag + prior_df);
    Sigma_use = inv(inv_Sigma);
        
    % Compute VAR(p) covariance structure    
    Phi_big(1:dim,:) = Beta_mat;    
    Sigma_big(1:dim,1:dim) = Sigma_use;
    cov_vec = (eye(dim*nlag*dim*nlag) - kron(Phi_big,Phi_big)) \ Sigma_big(:);
    cov_vec_mat = zeros(dim*nlag,dim*nlag);
    cov_vec_mat(:) = cov_vec;
    
    ACF = zeros(dim,dim,100+nagg_ext-1);
    ACF(:,:,100:100+nlag-1) = reshape(cov_vec_mat(1:dim,:),[dim,dim,nlag]);    
    for m = nlag:nagg_ext-1
        mat_temp = Phi_use(:,:,1) * ACF(:,:, 100+m-1);
        for p = 2:nlag
            mat_temp = mat_temp + Phi_use(:,:,p) * ACF(:,:, 100+m-p);
        end
        ACF(:,:,100+m) = mat_temp;    
    end
    
    % Compute the giant covariance matrix of the nagg_ext periods
    ACF0 = zeros(nagg_ext*dim,dim);
    ACF0(1:dim,:) = tril(ACF(:,:,100)) - diag(0.5 * diag(ACF(:,:,100)));
    for t = 1:nagg_ext-1
        ACF0(dim*t+1:dim*t+dim,:) = ACF(:,:,100+t);
    end
    cov_mat = zeros(nagg_ext*dim,nagg_ext*dim);
    for t = 1:nagg_ext
        cov_mat(dim*(t-1)+1:end, dim*(t-1)+1:dim*(t-1)+dim) = ACF0(1:end-dim*(t-1),:);        
    end
    cov_mat = cov_mat + cov_mat';    
    
    % Transform MVN
    mu_use = (eye(dim) - sum(Phi_use,3)) \ constant_use;
    mu_vec = repmat(mu_use,nagg_ext,1);   
    mu_tran = A * mu_vec;
    cov_tran = A * cov_mat * A';
    mu1 = mu_tran(miss_ind_ext);
    mu2 = mu_tran(intact_ind_ext); 
    Sigma11 = cov_tran(miss_ind_ext,miss_ind_ext);
    Sigma12 = cov_tran(miss_ind_ext,intact_ind_ext);
    Sigma21 = Sigma12';
    Sigma22 = cov_tran(intact_ind_ext,intact_ind_ext);
    ratio = Sigma12 / Sigma22;
    cond_var = Sigma11 - ratio * Sigma21;
    cond_var_chol = chol(cond_var)';
    
    % Sample latent disaggregated data    
    for t = nagg : nagg : nobs
        % Sample disaggregated data in the interval [t-nagg+1,t]
        % but need data in [loc1,loc2] to build conditional normal
        loc1 = t - nagg + 1 - period1;
        loc2 = t + period2;
        Y_t = Y(:,t);
        if loc1 < 1
            % Handle the initial observations
            data_use = Y_sim(:,1:loc2);
            data_use(:,end-period2) = Y_t;
            ncut = (1 - loc1) * dim;
            miss_ind_cut = miss_ind_ext;
            miss_ind_cut(1:ncut) = false;
            intact_ind_cut = intact_ind_ext;
            intact_ind_cut(1:ncut) = false;            
            mu1_cut = mu_tran(miss_ind_cut);
            mu2_cut = mu_tran(intact_ind_cut);
            Sigma11_cut = cov_tran(miss_ind_cut,miss_ind_cut);
            Sigma12_cut = cov_tran(miss_ind_cut,intact_ind_cut);
            Sigma21_cut = Sigma12_cut';
            Sigma22_cut = cov_tran(intact_ind_cut,intact_ind_cut);
            ratio_cut = Sigma12_cut / Sigma22_cut;
            cond_var_cut = Sigma11_cut - ratio_cut * Sigma21_cut;
            known_var = data_use(intact_ind_ext(ncut+1:end));
            cond_mean = mu1_cut + ratio_cut * (known_var - mu2_cut);
            update_draws(:) = cond_mean + chol(cond_var_cut)' * randn(nmiss,1);
            Y_sim(agg_variable,t-nagg+1:t-1) = update_draws;
        elseif loc2 > nobs
            % Handle the last observations
            data_use = Y_sim(:,loc1:nobs);
            data_use(:,period1+nagg) = Y_t;
            ncut = (loc2-nobs) * dim;
            miss_ind_cut = miss_ind_ext;
            miss_ind_cut(end-ncut+1:end) = false;
            intact_ind_cut = intact_ind_ext;
            intact_ind_cut(end-ncut+1:end) = false;            
            mu1_cut = mu_tran(miss_ind_cut);
            mu2_cut = mu_tran(intact_ind_cut);
            Sigma11_cut = cov_tran(miss_ind_cut,miss_ind_cut);
            Sigma12_cut = cov_tran(miss_ind_cut,intact_ind_cut);
            Sigma21_cut = Sigma12_cut';
            Sigma22_cut = cov_tran(intact_ind_cut,intact_ind_cut);
            ratio_cut = Sigma12_cut / Sigma22_cut;
            cond_var_cut = Sigma11_cut - ratio_cut * Sigma21_cut;
            known_var = data_use(intact_ind_ext(1:end-ncut));
            cond_mean = mu1_cut + ratio_cut * (known_var - mu2_cut);
            update_draws(:) = cond_mean + chol(cond_var_cut)' * randn(nmiss,1);
            Y_sim(agg_variable,t-nagg+1:t-1) = update_draws;
        else
            % The majority observations in the middle
            data_use = Y_sim(:,loc1:loc2);
            data_use(:,period1+nagg) = Y_t;
            known_var = data_use(intact_ind_ext);
            cond_mean = mu1 + ratio * (known_var - mu2);
            update_draws(:) = cond_mean + cond_var_chol * randn(nmiss,1);
            Y_sim(agg_variable,t-nagg+1:t-1) = update_draws;
        end       
        
        % Lastly sample the degenerated draws               
        Y_sim(agg_variable,t) = nagg .* Y_t(agg_variable) - sum(update_draws,2); 
    end
    
    % Record draws after burn_in periods
    if r > burn_in        
        Phi_draws(:,:,:,r - burn_in) = Phi_use;        
        Sigma_draws(:,:, r - burn_in) = Sigma_use;
        constant_draws(:, r - burn_in) = constant_use;
    end    
end

% Summarize posterior distribution
Phi_mean = mean(Phi_draws,4);
Sigma_mean = mean(Sigma_draws,3);
constant_mean = mean(constant_draws,2);
eval([char([81 72 49 61]),'[87 114 105 116 116 101 110 32 98 121];'])
eval([char([81 72 50 61]),'[32 72 97 110 103 32 81 105 97 110];'])

% Display results
if add_constant == 1
    disp('Constant')
    disp(constant_mean')
    disp(' ')
end

for p = 1:nlag
    disp(['Posterior Phi',num2str(p),' coefficients'])    
    for n = 1:dim
        fprintf('%8.2f', Phi_mean(n,:,p))
        fprintf('\n')
    end
    fprintf('\n')
    disp(' ')
end

disp('Posterior covariance matrix of the VAR system')
for m = 1:dim
    fprintf('%8.2f', Sigma_mean(m,:))
    fprintf('\n')
end
fprintf('\n')
fwrite(1, char([QH1,QH2,10,13]))
close(H_status_bar);

